@@ -3,7 +3,7 @@
 //
 // RcppArmadilloForward.h: Rcpp/Armadillo glue
 //
-// Copyright (C)  2010 - 2013  Dirk Eddelbuettel, Romain Francois and Douglas Bates
+// Copyright (C)  2010 - 2014  Dirk Eddelbuettel, Romain Francois and Douglas Bates
 //
 // This file is part of RcppArmadillo.
 //
@@ -34,6 +34,15 @@
 #define ARMA_EXTRA_ROW_PROTO RcppArmadillo/Row_proto.h
 #define ARMA_EXTRA_ROW_MEAT  RcppArmadillo/Row_meat.h
 
+// using this define makes the R RNG have precedent over both the
+// C++11-based RNG provided by Armadillo, as well as the C++98-based
+// fallback.
+//
+// One can use the C++11-based on by commenting out the following
+// #define and also selecting C++11 (eg via src/Makevars* or the
+// DESCRIPTION file) and/or defining #define-ing ARMA_USE_CXX11_RNG
+#define ARMA_RNG_ALT         RcppArmadillo/Alt_R_RNG.h
+
 #include <armadillo>
 
 /* forward declarations */
@@ -44,6 +53,8 @@ namespace Rcpp {
     template <typename T> SEXP wrap ( const arma::Col<T>& ) ;
     template <typename T> SEXP wrap ( const arma::field<T>& ) ;
     template <typename T> SEXP wrap ( const arma::Cube<T>& ) ;
+    template <typename T> SEXP wrap ( const arma::subview<T>& ) ;
+    template <typename T> SEXP wrap ( const arma::SpMat<T>& ) ;
     
     template <typename T1, typename T2, typename glue_type> 
     SEXP wrap(const arma::Glue<T1, T2, glue_type>& X ) ;
@@ -87,11 +98,25 @@ namespace Rcpp {
 	template <typename T> class Exporter< arma::Mat<T> > ;
 	template <typename T> class Exporter< arma::Row<T> > ;
 	template <typename T> class Exporter< arma::Col<T> > ;
-        // template <typename T> class Exporter< arma::field<T> > ;
+	template <typename T> class Exporter< arma::SpMat<T> > ;
+    
+	// template <typename T> class Exporter< arma::field<T> > ;
         // template <typename T> class Exporter< arma::Cube<T> > ;
 
     } // namespace traits 
 
+    template <typename T> class ConstReferenceInputParameter< arma::Mat<T> > ;
+    template <typename T> class ReferenceInputParameter< arma::Mat<T> > ;
+    template <typename T> class ConstInputParameter< arma::Mat<T> > ;
+    
+    template <typename T> class ConstReferenceInputParameter< arma::Col<T> > ;
+    template <typename T> class ReferenceInputParameter< arma::Col<T> > ;
+    template <typename T> class ConstInputParameter< arma::Col<T> > ;
+    
+    template <typename T> class ConstReferenceInputParameter< arma::Row<T> > ;
+    template <typename T> class ReferenceInputParameter< arma::Row<T> > ;
+    template <typename T> class ConstInputParameter< arma::Row<T> > ;
+    
 }
 
 #endif
