@@ -283,13 +283,94 @@ gmm_diag<eT>::save(const std::string name) const
 
 
 template<typename eT>
-template<typename T1>
 inline
-typename enable_if2<resolves_to_colvector<T1>::value, eT>::result
-gmm_diag<eT>::log_p(const T1& expr) const
+Col<eT>
+gmm_diag<eT>::generate() const
   {
   arma_extra_debug_sigprint();
   
+  const uword n_dims = means.n_rows;
+  const uword n_gaus = means.n_cols;
+  
+  Col<eT> out( (n_gaus > 0) ? n_dims : uword(0) );
+  
+  if(n_gaus > 0)
+    {
+    const double val = randu<double>();
+    
+    double csum    = double(0);
+    uword  gaus_id = 0;
+    
+    for(uword j=0; j < n_gaus; ++j)
+      {
+      csum += hefts[j];
+      
+      if(val <= csum)  { gaus_id = j; break; }
+      }
+    
+    out =  randn< Col<eT> >(n_dims);    
+    out %= sqrt(dcovs.col(gaus_id));
+    out += means.col(gaus_id);
+    }
+  
+  return out;
+  }
+
+
+
+template<typename eT>
+inline
+Mat<eT>
+gmm_diag<eT>::generate(const uword N) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const uword n_dims = means.n_rows;
+  const uword n_gaus = means.n_cols;
+  
+  Mat<eT> out( ( (n_gaus > 0) ? n_dims : uword(0) ), N );
+  
+  if(n_gaus > 0)
+    {
+    const eT* hefts_mem = hefts.memptr();
+    
+    for(uword i=0; i < N; ++i)
+      {
+      const double val = randu<double>();
+      
+      double csum    = double(0);
+      uword  gaus_id = 0;
+      
+      for(uword j=0; j < n_gaus; ++j)
+        {
+        csum += hefts_mem[j];
+        
+        if(val <= csum)  { gaus_id = j; break; }
+        }
+      
+      subview_col<eT> out_col = out.col(i);
+      
+      out_col =  randn< Col<eT> >(n_dims);    
+      out_col %= sqrt(dcovs.col(gaus_id));
+      out_col += means.col(gaus_id);
+      }
+    }
+  
+  return out;
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+eT
+gmm_diag<eT>::log_p(const T1& expr, const gmm_empty_arg& junk1, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true))>::result* junk2) const
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk1);
+  arma_ignore(junk2);
+  
   const quasi_unwrap<T1> tmp(expr);
   
   arma_debug_check( (tmp.M.n_rows != means.n_rows), "gmm_diag::log_p(): incompatible dimensions" );
@@ -302,10 +383,11 @@ gmm_diag<eT>::log_p(const T1& expr) const
 template<typename eT>
 template<typename T1>
 inline
-typename enable_if2<resolves_to_colvector<T1>::value, eT>::result
-gmm_diag<eT>::log_p(const T1& expr, const uword gaus_id) const
+eT
+gmm_diag<eT>::log_p(const T1& expr, const uword gaus_id, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true))>::result* junk2) const
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk2);
   
   const quasi_unwrap<T1> tmp(expr);
   
@@ -321,10 +403,12 @@ gmm_diag<eT>::log_p(const T1& expr, const uword gaus_id) const
 template<typename eT>
 template<typename T1>
 inline
-typename enable_if2< is_arma_type<T1>::value && (resolves_to_colvector<T1>::value == false), Row<eT> >::result
-gmm_diag<eT>::log_p(const T1& expr) const
+Row<eT>
+gmm_diag<eT>::log_p(const T1& expr, const gmm_empty_arg& junk1, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk2) const
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk1);
+  arma_ignore(junk2);
   
   if(is_subview<T1>::value)
     {
@@ -346,10 +430,11 @@ gmm_diag<eT>::log_p(const T1& expr) const
 template<typename eT>
 template<typename T1>
 inline
-typename enable_if2< is_arma_type<T1>::value && (resolves_to_colvector<T1>::value == false), Row<eT> >::result
-gmm_diag<eT>::log_p(const T1& expr, const uword gaus_id) const
+Row<eT>
+gmm_diag<eT>::log_p(const T1& expr, const uword gaus_id, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk2) const
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk2);
   
   if(is_subview<T1>::value)
     {
@@ -421,10 +506,11 @@ gmm_diag<eT>::avg_log_p(const Base<eT,T1>& expr, const uword gaus_id) const
 template<typename eT>
 template<typename T1>
 inline
-typename enable_if2<resolves_to_colvector<T1>::value, uword>::result
-gmm_diag<eT>::assign(const T1& expr, const gmm_dist_mode& dist) const
+uword
+gmm_diag<eT>::assign(const T1& expr, const gmm_dist_mode& dist, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true))>::result* junk) const
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
   
   if(is_subview_col<T1>::value)
     {
@@ -446,10 +532,11 @@ gmm_diag<eT>::assign(const T1& expr, const gmm_dist_mode& dist) const
 template<typename eT>
 template<typename T1>
 inline
-typename enable_if2<is_arma_type<T1>::value && (resolves_to_colvector<T1>::value == false), urowvec>::result
-gmm_diag<eT>::assign(const T1& expr, const gmm_dist_mode& dist) const
+urowvec
+gmm_diag<eT>::assign(const T1& expr, const gmm_dist_mode& dist, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk) const
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
   
   urowvec out;
   
@@ -535,11 +622,12 @@ gmm_diag<eT>::norm_hist(const Base<eT,T1>& expr, const gmm_dist_mode& dist_mode)
 
 
 template<typename eT>
+template<typename T1>
 inline
 bool
 gmm_diag<eT>::learn
   (
-  const Mat<eT>&       X,
+  const Base<eT,T1>&   data,
   const uword          n_gaus,
   const gmm_dist_mode& dist_mode,
   const gmm_seed_mode& seed_mode,
@@ -564,6 +652,9 @@ gmm_diag<eT>::learn
   arma_debug_check( (seed_mode_ok == false), "gmm_diag::learn(): unknown seed_mode"                        );
   arma_debug_check( (var_floor < eT(0)    ), "gmm_diag::learn(): variance floor is negative"               );
   
+  const unwrap<T1>   tmp_X(data.get_ref());
+  const Mat<eT>& X = tmp_X.M;
+  
   if(X.is_empty()          )  { arma_warn(true, "gmm_diag::learn(): given matrix is empty"             ); reset(); return false; }
   if(X.is_finite() == false)  { arma_warn(true, "gmm_diag::learn(): given matrix has non-finite values"); reset(); return false; }
   
