@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2011 Conrad Sanderson
-// Copyright (C) 2008-2011 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2014 Conrad Sanderson
+// Copyright (C) 2008-2014 NICTA (www.nicta.com.au)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,6 +10,8 @@
 //! @{
 
 
+struct mtOp_dual_aux_indicator {};
+
 
 template<typename out_eT, typename T1, typename op_type>
 class mtOp : public Base<out_eT, mtOp<out_eT, T1, op_type> >
@@ -21,8 +23,8 @@ class mtOp : public Base<out_eT, mtOp<out_eT, T1, op_type> >
 
   typedef typename T1::elem_type                in_eT;
 
-  static const bool is_row = T1::is_row && is_op_mixed_elem<op_type>::value;
-  static const bool is_col = T1::is_col && is_op_mixed_elem<op_type>::value; 
+  static const bool is_row = T1::is_row && (is_op_mixed_elem<op_type>::value || is_same_type<op_type, op_clamp>::value);
+  static const bool is_col = T1::is_col && (is_op_mixed_elem<op_type>::value || is_same_type<op_type, op_clamp>::value);
   
   inline explicit mtOp(const T1& in_m);
   inline          mtOp(const T1& in_m, const in_eT in_aux);
@@ -31,6 +33,8 @@ class mtOp : public Base<out_eT, mtOp<out_eT, T1, op_type> >
   
   inline          mtOp(const char junk, const T1& in_m, const out_eT in_aux);
   
+  inline          mtOp(const mtOp_dual_aux_indicator&, const T1& in_m, const in_eT in_aux_a, const out_eT in_aux_b);
+  
   inline         ~mtOp();
     
   
